/**
 * Get a timer value in milliseconds resolution with Date class.
 *
 * @return  A timer value in milliseconds.
 *
 * @since 0.2.5
 * @internal
 */
export declare function nowDate(): number;
/**
 * Get a timer value in microseconds resolution with Performance.now function.
 *
 * @return  A timer value in milliseconds. (microseconds resolution)
 *
 * @since 0.2.5
 * @internal
 */
export declare function nowNow(): number;
/**
 * Get a timer value in nanoseconds resolution with Process.hrtime function.
 *
 * @return  A timer value in milliseconds. (nanoseconds resolution)
 *
 * @since 0.2.5
 * @internal
 */
export declare function nowHrtime(): number;
/**
 * Get the current time as high resolution as possible in the current platform.
 *
 * @return  A timer value in milliseconds.
 *
 * @internal
 */
export declare const now: typeof nowHrtime;
/**
 * Measure tiem to execute a function.
 *
 * wait for done if the target function returns a thenable object. so you can use async function.
 *
 * **NOTE:** this function will execute target function only once.
 *
 * @param fun      The target function.
 * @param args     Arguments to passing to target function.
 * @param context  The `this` for target function.
 *
 * @return milliseconds taked executing.
 *
 * ## Examples
 * ### Simple usage
 * ``` typescript
 * const msec = await timeit(function() {
 *     # do something heavy.
 * });
 * ```
 *
 * ### With arguments
 * ``` typescript
 * console.log(await timeit(axios.get, ['http://example.com']));
 * ```
 *
 * ## Changelog
 *
 * - 1.0.0: Changed arguments order.
 *
 * @since 0.2.4
 */
export declare function timeit<T extends unknown[], U extends Record<string, unknown>>(fun: ((...args: T) => Promise<void> | void), args?: T, context?: U): Promise<number>;
