(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
    typeof define === 'function' && define.amd ? define(['exports'], factory) :
    (global = global || self, factory(global.AsyncMark = {}));
}(this, (function (exports) { 'use strict';

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation.

    Permission to use, copy, modify, and/or distribute this software for any
    purpose with or without fee is hereby granted.

    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
    PERFORMANCE OF THIS SOFTWARE.
    ***************************************************************************** */
    /* global Reflect, Promise */

    var extendStatics = function(d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };

    function __extends(d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    }

    var __assign = function() {
        __assign = Object.assign || function __assign(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
            }
            return t;
        };
        return __assign.apply(this, arguments);
    };

    function __awaiter(thisArg, _arguments, P, generator) {
        function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
        return new (P || (P = Promise))(function (resolve, reject) {
            function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
            function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
            function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
            step((generator = generator.apply(thisArg, _arguments || [])).next());
        });
    }

    function __generator(thisArg, body) {
        var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
        return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
        function verb(n) { return function (v) { return step([n, v]); }; }
        function step(op) {
            if (f) throw new TypeError("Generator is already executing.");
            while (_) try {
                if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
                if (y = 0, t) op = [op[0] & 2, t.value];
                switch (op[0]) {
                    case 0: case 1: t = op; break;
                    case 4: _.label++; return { value: op[1], done: false };
                    case 5: _.label++; y = op[1]; op = [0]; continue;
                    case 7: op = _.ops.pop(); _.trys.pop(); continue;
                    default:
                        if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                        if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                        if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                        if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                        if (t[2]) _.ops.pop();
                        _.trys.pop(); continue;
                }
                op = body.call(thisArg, _);
            } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
            if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
        }
    }

    function __spreadArrays() {
        for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
        for (var r = Array(s), k = 0, i = 0; i < il; i++)
            for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
                r[k] = a[j];
        return r;
    }

    /**
     * Error class for benchmark result assertion.
     *
     * @since 1.0.0
     */
    var AsyncMarkAssertionError = /** @class */ (function (_super) {
        __extends(AsyncMarkAssertionError, _super);
        /**
         * @param rule          The rule that couses error.
         * @param result        The result for assert.
         * @param stackStartFn  Provided function will remove from stack trace.
         *
         * @internal
         */
        function AsyncMarkAssertionError(rule, result, stackStartFn) {
            var _newTarget = this.constructor;
            var _this = _super.call(this, "benchmark \"" + result.name + "\": actual:" + result.average + "msec/op " + rule.operator + " expected:" + rule.expected + "msec/op") || this;
            /**
             * @internal
             */
            _this.name = 'AsyncMarkAssertionError';
            _this.actual = result.average;
            _this.expected = rule.expected;
            _this.operator = rule.operator;
            Object.setPrototypeOf(_this, _newTarget.prototype);
            if (Error.captureStackTrace !== undefined && stackStartFn !== undefined) {
                Error.captureStackTrace(_this, stackStartFn);
            }
            return _this;
        }
        return AsyncMarkAssertionError;
    }(Error));

    /**
     * Convert unit to number
     *
     * @param u  unit name like 'ms', 'sec' or etc.
     *
     * @return  number to convert milliseconds.
     *
     * ## Examples
     * ``` typescript
     * assert(100 * unit('ms') == 0.1 * unit('sec'))
     * ```
     *
     * @since 0.3.0
     * @internal
     */
    function unit(u) {
        switch (u) {
            case 's':
            case 'sec':
                return 1e3;
            case '':
            case 'ms':
            case 'msec':
                return 1;
            case 'us':
            case 'usec':
                return 1e-3;
            case 'ns':
            case 'nsec':
                return 1e-6;
            default:
                throw Error("unknown unit name: \"" + u + "\"");
        }
    }
    /**
     * The assertion rule
     *
     * @since 0.3.0
     * @internal
     */
    var AssertRule = /** @class */ (function () {
        /**
         * Parse time rule for assertion.
         *
         * Rule format is `{operator}{number}{unit}`; use like `<=10msec`.
         * Operator and unit are can omit. If omitted, uses `<=` and `msec`.
         *
         * @param rule  Assert rule that milliseconds {@link Number} or {@link String} value
         *              like '<10ms' or '>=20s'.
         *
         * @return
         *
         * ## Supported operators
         * |example       |means              |
         * |--------------|-------------------|
         * |`<42`         |faster than 42 msec|
         * |`<=42` or omit|42 msec or faster  |
         * |`>42`         |slower than 42 msec|
         * |`>=42`        |42 msec or slower  |
         *
         * ## Supported units
         * |example           |means       |
         * |------------------|------------|
         * |`42s` or `42sec`  |seconds     |
         * |`42ms` or `42msec`|milliseconds|
         * |`42us` or `42usec`|microseconds|
         * |`42ns` or `42nsec`|nanoseconds |
         */
        function AssertRule(rule) {
            var m = String(rule).match(/^(|[<>]=?)(\d+(?:\.\d+)?)(|s|ms|us|ns|sec|msec|usec|nsec)$/);
            if (m === null) {
                throw Error("Invalid rule format: \"" + rule + "\"");
            }
            this.operator = (m[1] || '<=');
            this.expected = Number(m[2]) * unit(m[3]);
        }
        /**
         * Checking benchmark result.
         *
         * @param msec  Target milliseconds.
         *
         * @return  Returns true if msec is acceptable.
         */
        AssertRule.prototype.check = function (msec) {
            var _this = this;
            return {
                '<': function (ms) { return ms < _this.expected; },
                '<=': function (ms) { return ms <= _this.expected; },
                '>': function (ms) { return ms > _this.expected; },
                '>=': function (ms) { return ms >= _this.expected; },
            }[this.operator](msec);
        };
        /**
         * Assert with benchmark result.
         *
         * @param result        Result of benchmark.
         * @param stackStartFn  Provided function will remove from stack trace.
         *
         * @throws  When result is unacceptable.
         *
         * @since 0.3.0
         */
        AssertRule.prototype.assert = function (result, stackStartFn) {
            if (!this.check(result.average)) {
                throw new AsyncMarkAssertionError(this, result, stackStartFn);
            }
        };
        return AssertRule;
    }());

    /**
     * The result of benchmark.
     *
     * This value will included outlier. Please use {@link Result.dropOutlier} if you want drop they.
     */
    var Result = /** @class */ (function () {
        /**
         * @param name   Name of benchmark.
         * @param msecs  Times of benchmark result.
         */
        function Result(name, msecs) {
            this.name = name;
            this.msecs = msecs;
        }
        Object.defineProperty(Result.prototype, "total", {
            /**
             * Total milliseconds of this benchmark.
             */
            get: function () {
                return this.msecs.reduce(function (x, y) { return x + y; });
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(Result.prototype, "fastest", {
            /**
             * The time of fastest test in milliseconds.
             */
            get: function () {
                return Math.min.apply(Math, this.msecs);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(Result.prototype, "slowest", {
            /**
             * The time of slowest test in milliseconds.
             */
            get: function () {
                return Math.max.apply(Math, this.msecs);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(Result.prototype, "average", {
            /**
             * Average time of this benchmark in milliseconds.
             */
            get: function () {
                return this.total / this.msecs.length;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(Result.prototype, "variance", {
            /**
             * Time unbiased sample variance of times.
             */
            get: function () {
                var avg = this.average;
                return this.msecs.reduce(function (total, x) { return total + Math.pow((x - avg), 2); }, 0) / (this.msecs.length - 1);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(Result.prototype, "std", {
            /**
             * Standard division of times.
             */
            get: function () {
                return Math.sqrt(this.variance);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(Result.prototype, "sem", {
            /**
             * Standard error of the mean of times.
             */
            get: function () {
                return this.std / Math.sqrt(this.msecs.length);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(Result.prototype, "errorRange", {
            /**
             * Guessed error range of this benchmark.
             */
            get: function () {
                return this.sem * 1.96;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(Result.prototype, "errorRate", {
            /**
             * Error range per average time.
             */
            get: function () {
                return this.errorRange / this.average;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(Result.prototype, "opsPerSec", {
            /**
             * Operations per seconds.
             */
            get: function () {
                return 1000 / this.average;
            },
            enumerable: false,
            configurable: true
        });
        /**
         * Make new Result that droped outlier.
         *
         * @param threshold  The threshold of outlier testing.
         *
         * @return  New {@link Result} instance.
         */
        Result.prototype.dropOutlier = function (threshold) {
            if (threshold === void 0) { threshold = 2; }
            var _a = this, average = _a.average, std = _a.std;
            return new Result(this.name, this.msecs.filter(function (x) { return Math.abs((x - average) / std) <= threshold; }));
        };
        /**
         * Convert to string for printing.
         *
         * @return  Human redable string
         */
        Result.prototype.toString = function () {
            var avg = Math.round(this.average * 10000) / 10000;
            var ops = Math.round(this.opsPerSec * 1000) / 1000;
            var range = Math.round(this.errorRange * 10000) / 10000;
            var rate = Math.round(this.errorRate * 10000) / 100;
            return this.name + ":\t" + ops + "ops/sec\t" + avg + "msec/op\t+-" + range + "msec/op (" + rate + "%)\t" + this.msecs.length + " times tried";
        };
        /**
         * Assertion if it taked more (or less) time than expected.
         *
         * Expected rule format is `{operator}{number}{unit}`; use like `<=10msec`.
         * Operator and unit are can omit. If omitted, uses `<=` and `msec`.
         *
         * This function will throw {@link AsyncMarkAssertionError} if result is unaccepcable.
         *
         * @param expected  Expected time in milliseconds number or string value.
         *                  e.g. `<10ms` or `>=20s`.
         *
         * @return
         *
         * ## Supported operators
         * |example       |means              |
         * |--------------|-------------------|
         * |"<42"         |faster than 42 msec|
         * |"<=42" or omit|42 msec or faster  |
         * |">42"         |slower than 42 msec|
         * |">=42"        |42 msec or slower  |
         *
         * ## Supported units
         * |example           |means       |
         * |------------------|------------|
         * |"42s" or "42sec"  |seconds     |
         * |"42ms" or "42msec"|milliseconds|
         * |"42us" or "42usec"|microseconds|
         * |"42ns" or "42nsec"|nanoseconds |
         *
         * ## Examples
         * ### Simple usage
         *
         * ``` typescript
         * const result = await Benchmark(function() {
         *     # do something that expect done in least 100msec
         * }).run();
         *
         * result.assert(100);
         * ```
         *
         * ### Combination rule
         *
         * ``` typescript
         * const result = await Benchmark(async function() {
         *     await sleep_function(100);
         * }).run();
         *
         * result.assert('>90ms', '<110ms');
         * ```
         *
         * @since 0.3.0
         */
        Result.prototype.assert = function () {
            var _this = this;
            var expected = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                expected[_i] = arguments[_i];
            }
            var rules = expected.map(function (x) { return new AssertRule(x); });
            rules.forEach(function (rule) { return rule.assert(_this, _this.assert); });
        };
        return Result;
    }());

    /**
     * Get a timer value in milliseconds resolution with Date class.
     *
     * @return  A timer value in milliseconds.
     *
     * @since 0.2.5
     * @internal
     */
    function nowDate() {
        return Number(new Date());
    }
    /**
     * Get a timer value in microseconds resolution with Performance.now function.
     *
     * @return  A timer value in milliseconds. (microseconds resolution)
     *
     * @since 0.2.5
     * @internal
     */
    function nowNow() {
        return performance.now();
    }
    /**
     * Get a timer value in nanoseconds resolution with Process.hrtime function.
     *
     * @return  A timer value in milliseconds. (nanoseconds resolution)
     *
     * @since 0.2.5
     * @internal
     */
    function nowHrtime() {
        var hr = process.hrtime();
        return (hr[0] * 1e9 + hr[1]) / 1e6;
    }
    /**
     * Get the current time as high resolution as possible in the current platform.
     *
     * @return  A timer value in milliseconds.
     *
     * @internal
     */
    var now = (function () {
        if (typeof process !== 'undefined' && process.hrtime) {
            return nowHrtime;
        }
        if (typeof performance !== 'undefined' && performance.now) {
            return nowNow;
        }
        return nowDate;
    })();
    /**
     * Measure tiem to execute a function.
     *
     * wait for done if the target function returns a thenable object. so you can use async function.
     *
     * **NOTE:** this function will execute target function only once.
     *
     * @param fun      The target function.
     * @param args     Arguments to passing to target function.
     * @param context  The `this` for target function.
     *
     * @return milliseconds taked executing.
     *
     * ## Examples
     * ### Simple usage
     * ``` typescript
     * const msec = await timeit(function() {
     *     # do something heavy.
     * });
     * ```
     *
     * ### With arguments
     * ``` typescript
     * console.log(await timeit(axios.get, ['http://example.com']));
     * ```
     *
     * ## Changelog
     *
     * - 1.0.0: Changed arguments order.
     *
     * @since 0.2.4
     */
    function timeit(fun, args, context) {
        if (args === void 0) { args = []; }
        if (context === void 0) { context = {}; }
        return __awaiter(this, void 0, void 0, function () {
            var start, end;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        start = now();
                        return [4 /*yield*/, fun.call.apply(fun, __spreadArrays([context], args))];
                    case 1:
                        _a.sent();
                        end = now();
                        return [2 /*return*/, end - start];
                }
            });
        });
    }

    /**
     * Class for benchmarking.
     *
     * Benchmark will execute by flow like this.
     *
     * - {@link Benchmark.before | before}
     * - {@link Benchmark.beforeEach | beforeEach}
     * - {@link Benchmark.fun | fun}
     * - {@link Benchmark.afterEach | afterEach}
     * - {@link Benchmark.after | after}
     *
     * Each function can override with options of the constructor.
     *
     *
     * ``` typescript
     * import Benchmark from 'asyncmark';
     *
     *
     * new Benchmark({
     *     name: 'timeout',
     *     fun() {
     *         return new Promise((resolve, reject) => {
     *             setTimeout(resolve, 100);
     *         });
     *     },
     * }).run().catch(console.error);
     * ```
     */
    var Benchmark = /** @class */ (function () {
        /**
         * @param options  The options for this benchmark or benchmarking function.
         */
        function Benchmark(options) {
            var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
            /**
             * Name of this benchmark.
             */
            this.name = 'unnamed';
            /**
             * Wanted maximum error rate.
             * This value will be ignore if set {@link Benchmark.number}.
             */
            this.targetErrorRate = 0.1;
            /**
             * Maximum number of executing test.
             * This value will be ignore if set {@link Benchmark.number}.
             */
            this.maxNumber = 10000;
            /**
             * Minimal number of executing test.
             * This value will be ignore if set {@link Benchmark.number}.
             */
            this.minNumber = 30;
            /**
             * The number of executing the test.
             * Will decide automatically in between {@link Benchmark.minNumber} to {@link Benchmark.maxNumber}
             * if set null.
             */
            this.number = null;
            /**
             * Setup before execute benchmark.
             *
             * At the time executing this method, `this` is the unique object for the benchmark.
             * So you can use `this` for storing testing data like a database.
             * Data of `this` that set in this method will discard after call {@link Benchmark.after}
             *
             * In default, do nothing.
             */
            this.before = function () { return undefined; };
            /**
             * Setup before each tests.
             *
             * At the time executing this method, `this` is the unique object for the test.
             * So you can use `this` for storing testing data.
             * Data of `this` that set in this method will discard after call {@link Benchmark.afterEach}
             *
             * In default, do nothing.
             */
            this.beforeEach = function () { return undefined; };
            /**
             * The target function for benchmarking.
             *
             * At the time executing this method, `this` is the unique object for the test.
             * So you can use `this` for storing testing data.
             * Data of `this` that set in this method will discard after call {@link Benchmark.afterEach}
             *
             * In default, couses error that `Error('target function is not defined')`.
             */
            this.fun = function () {
                throw new Error('target function is not defined');
            };
            /**
             * Teardown after each tests.
             *
             * At the time executing this method, `this` is the unique object for the test.
             * So you can use `this` for storing testing data.
             * Data of `this` that set in this method will discard after call this method.
             *
             * In default, do nothing.
             */
            this.afterEach = function () { return undefined; };
            /**
             * Teardown after execute benchmark.
             *
             * At the time executing this method, `this` is the unique object for the benchmark.
             * So you can use `this` for storing testing data like a database.
             * Data of `this` that set in this method will discard after call this method.
             *
             * In default, shows test result.
             */
            this.after = function (result) {
                /* eslint-disable-next-line no-console */
                console.log(String(result.dropOutlier()));
            };
            if (typeof options === 'function') {
                this.fun = options;
                return;
            }
            this.name = (_a = options.name) !== null && _a !== void 0 ? _a : this.name;
            this.targetErrorRate = (_b = options.targetErrorRate) !== null && _b !== void 0 ? _b : this.targetErrorRate;
            this.maxNumber = (_c = options.maxNumber) !== null && _c !== void 0 ? _c : this.maxNumber;
            this.minNumber = (_d = options.minNumber) !== null && _d !== void 0 ? _d : this.minNumber;
            this.number = (_e = options.number) !== null && _e !== void 0 ? _e : this.number;
            this.before = (_f = options.before) !== null && _f !== void 0 ? _f : this.before;
            this.beforeEach = (_g = options.beforeEach) !== null && _g !== void 0 ? _g : this.beforeEach;
            this.fun = (_h = options.fun) !== null && _h !== void 0 ? _h : this.fun;
            this.afterEach = (_j = options.afterEach) !== null && _j !== void 0 ? _j : this.afterEach;
            this.after = (_k = options.after) !== null && _k !== void 0 ? _k : this.after;
        }
        /**
         * Execute benchmark.
         *
         * @param context    The `this` for each benchmarking functions.
         *                   `__proto__` will override with this instance.
         * @param callbacks  Callback functions.
         *
         * @return A result of benchmark.
         */
        Benchmark.prototype.run = function (context, _a) {
            if (context === void 0) { context = {}; }
            var _b = _a === void 0 ? {} : _a, _c = _b.beforeTest, beforeTest = _c === void 0 ? function () { return undefined; } : _c, _d = _b.afterTest, afterTest = _d === void 0 ? function () { return undefined; } : _d;
            return __awaiter(this, void 0, void 0, function () {
                var ctxInTest, loopNum, msecs, i, ctxInLoop, msec, result;
                return __generator(this, function (_e) {
                    switch (_e.label) {
                        case 0:
                            ctxInTest = __assign({}, context);
                            ctxInTest.__proto__ = this;
                            return [4 /*yield*/, this.before.call(ctxInTest)];
                        case 1:
                            _e.sent();
                            loopNum = this.number || this.maxNumber;
                            msecs = [];
                            i = 0;
                            _e.label = 2;
                        case 2:
                            if (!(i < loopNum)) return [3 /*break*/, 9];
                            ctxInLoop = __assign({}, ctxInTest);
                            return [4 /*yield*/, beforeTest.call(ctxInLoop, i, this)];
                        case 3:
                            _e.sent();
                            return [4 /*yield*/, this.beforeEach.call(ctxInLoop, i)];
                        case 4:
                            _e.sent();
                            return [4 /*yield*/, timeit(this.fun, [], ctxInLoop)];
                        case 5:
                            msec = _e.sent();
                            msecs.push(msec);
                            return [4 /*yield*/, this.afterEach.call(ctxInLoop, i, msec)];
                        case 6:
                            _e.sent();
                            return [4 /*yield*/, afterTest.call(ctxInLoop, i, this, msec)];
                        case 7:
                            _e.sent();
                            if (!this.number
                                && i + 1 >= this.minNumber
                                && new Result(this.name, msecs).errorRate <= this.targetErrorRate) {
                                return [3 /*break*/, 9];
                            }
                            _e.label = 8;
                        case 8:
                            i += 1;
                            return [3 /*break*/, 2];
                        case 9:
                            result = new Result(this.name, msecs);
                            return [4 /*yield*/, this.after.call(ctxInTest, result)];
                        case 10:
                            _e.sent();
                            return [2 /*return*/, result];
                    }
                });
            });
        };
        return Benchmark;
    }());

    /**
     * A set of {@link Benchmark}s for executing those sequential or parallel.
     *
     * Suite will execute by flow like this.
     *
     * - {@link Suite.before}
     * - {@link Suite.beforeEach}
     * - {@link Benchmark.before}
     * - {@link Suite.beforeTest}
     * - {@link Benchmark.beforeEach}
     * - {@link Benchmark.fun}
     * - {@link Benchmark.afterEach}
     * - {@link Suite.afterTest}
     * - {@link Benchmark.after}
     * - {@link Suite.afterEach}
     * - {@link Suite.after}
     *
     * Each function can override with options of the constructor.
     *
     *
     * ## Example
     * ``` typescript
     * import {Suite} from 'asyncmark';
     *
     *
     * const suite = new Suite({
     *     name: 'ways to find a character',
     *     beforeEach() {
     *         this.text = 'hello world';
     *     },
     *     parallel: true,
     * });
     *
     * suite.add(function() {
     *     /o/.test(this.text);
     * });
     *
     * suite.add({
     *     name: 'String#indexOf',
     *     before() {
     *         console.log('starting String#indexOf...');
     *     },
     *     fun() {
     *         this.text.indexOf('o') > -1;
     *     },
     * });
     *
     * suite.add(new Benchmark({
     *     name: 'String#match',
     *     fun() {
     *         Boolean(this.text.match(/o/));
     *     },
     *     after(result) {
     *         console.log('String#match is done! ' + result);
     *     },
     * }));
     *
     * suite.run()
     *     .then(results => {
     *         let min = results[0];
     *         results.forEach(x => {
     *             if (min.average > x.average) {
     *                 min = x;
     *             }
     *         });
     *         console.log(min.name + ' is best way!');
     *     }).
     *     catch(err => console.error(err));
     * ```
     */
    var Suite = /** @class */ (function () {
        /**
         * @param options  Options for this suite.
         */
        function Suite(options) {
            if (options === void 0) { options = {}; }
            var _a, _b, _c, _d, _e, _f, _g, _h, _j;
            /**
             * Name of this suite.
             */
            this.name = 'unnamed';
            /**
             * Default options for benchmarks in this suite.
             */
            this.benchmarkDefault = {};
            /**
             * Flag for executing each benchmark parallelly.
             */
            this.parallel = false;
            /**
             * A list of {@link Benchmark} or {@link Suite}.
             */
            this.benchmarks = [];
            /**
             * Setup before execute all benchmarks.
             *
             * At the time executing this method, `this` is the unique object for the suite.
             * So you can use `this` for storing testing data like a database.
             * Data of `this` that set in this method will discard after call {@link Suite.after}.
             *
             * In default, do nothing.
             */
            this.before = function () { return undefined; };
            /**
             * Setup before execute each child {@link Benchmark} or {@link Suite}.
             *
             * At the time executing this method, `this` is the unique object for the benchmark.
             * So you can use `this` for storing testing data like a database.
             * Data of `this` that set in this method will discard after call {@link Suite.afterEach}.
             *
             * In default, do nothing.
             */
            this.beforeEach = function () { return undefined; };
            /**
             * Setup before execute each test of benchmarks.
             *
             * At the time executing this method, `this` is the unique object for the test.
             * So you can use `this` for storing testing data like a database.
             * Data of `this` that set in this method will discard after call {@link Suite.afterTest}.
             *
             * In default, do nothing.
             */
            this.beforeTest = function () { return undefined; };
            /**
             * Teardown after execute each test of benchmarks.
             *
             * At the time executing this method, `this` is the unique object for the test.
             * So you can use `this` for storing testing data like a database.
             * Data of `this` that set in this method will discard after call this method
             *
             * In default, do nothing.
             */
            this.afterTest = function () { return undefined; };
            /**
             * Teardown after execute each benchmark.
             *
             * At the time executing this method, `this` is the unique object for the benchmark.
             * So you can use `this` for storing testing data like a database.
             * Data of `this` that set in this method will discard after call this method.
             *
             * In default, do nothing.
             */
            this.afterEach = function () { return undefined; };
            /**
             * Teardown after execute all benchmarks.
             *
             * At the time executing this method, `this` is the unique object for the suite.
             * So you can use `this` for storing testing data like a database.
             * Data of `this` that set in this method will discard after call this method.
             *
             * In default, do nothing.
             */
            this.after = function () { return undefined; };
            this.name = (_a = options.name) !== null && _a !== void 0 ? _a : this.name;
            this.benchmarkDefault = (_b = options.benchmarkDefault) !== null && _b !== void 0 ? _b : {};
            this.parallel = (_c = options.parallel) !== null && _c !== void 0 ? _c : this.parallel;
            this.before = (_d = options.before) !== null && _d !== void 0 ? _d : this.before;
            this.beforeEach = (_e = options.beforeEach) !== null && _e !== void 0 ? _e : this.beforeEach;
            this.beforeTest = (_f = options.beforeTest) !== null && _f !== void 0 ? _f : this.beforeTest;
            this.afterTest = (_g = options.afterTest) !== null && _g !== void 0 ? _g : this.afterTest;
            this.afterEach = (_h = options.afterEach) !== null && _h !== void 0 ? _h : this.afterEach;
            this.after = (_j = options.after) !== null && _j !== void 0 ? _j : this.after;
        }
        /**
         * Adding {@link Benchmark} instance into this {@link Suite}.
         *
         * @param benchmark  The benchmark instance for adding.
         *
         * @return  Returns this suite for method chain.
         */
        Suite.prototype.addBenchmark = function (benchmark) {
            this.benchmarks.push(benchmark);
            return this;
        };
        /**
         * Adding child {@link Suite} instance into this {@link Suite}.
         *
         * @param suite  The suite instance for adding.
         *
         * @return  Returns this suite for method chain.
         */
        Suite.prototype.addSuite = function (suite) {
            this.benchmarks.push(suite);
            return this;
        };
        /**
         * Make new benchmark or suite and adding into this {@link Suite}.
         *
         * @param child  {@link Benchmark}, {@link Suite} or arguments for {@link Benchmark.constructor}.
         *
         * @return  Returns this suite for method chain.
         */
        Suite.prototype.add = function (child) {
            if (child instanceof Benchmark) {
                this.addBenchmark(child);
            }
            else if (child instanceof Suite) {
                this.addSuite(child);
            }
            else if (typeof child === 'function') {
                var options = { fun: child };
                options.__proto__ = this.benchmarkDefault;
                this.addBenchmark(new Benchmark(options));
            }
            else {
                var options = __assign({}, child);
                options.__proto__ = this.benchmarkDefault;
                this.addBenchmark(new Benchmark(options));
            }
            return this;
        };
        /**
         * Make callbacks for {@link Benchmark.run}.
         *
         * @param count            Count of benchmark in this suite.
         * @param parentCallbacks  Callback functions of parent suite.
         *                         Same as callbacks of {@link Suite.run}.
         *
         * @return  Callbacks.
         */
        Suite.prototype._makeCallbacks = function (count, parentCallbacks) {
            var _a = this, beforeTest = _a.beforeTest, afterTest = _a.afterTest;
            return {
                beforeTest: function (c, b) {
                    return __awaiter(this, void 0, void 0, function () {
                        return __generator(this, function (_a) {
                            switch (_a.label) {
                                case 0:
                                    if (!parentCallbacks.beforeTest) return [3 /*break*/, 2];
                                    return [4 /*yield*/, parentCallbacks.beforeTest.call(this, c, b)];
                                case 1:
                                    _a.sent();
                                    _a.label = 2;
                                case 2: return [4 /*yield*/, beforeTest.call(this, count, c, b)];
                                case 3:
                                    _a.sent();
                                    return [2 /*return*/];
                            }
                        });
                    });
                },
                afterTest: function (c, b, r) {
                    return __awaiter(this, void 0, void 0, function () {
                        return __generator(this, function (_a) {
                            switch (_a.label) {
                                case 0: return [4 /*yield*/, afterTest.call(this, count, c, b, r)];
                                case 1:
                                    _a.sent();
                                    if (!parentCallbacks.afterTest) return [3 /*break*/, 3];
                                    return [4 /*yield*/, parentCallbacks.afterTest.call(this, c, b, r)];
                                case 2:
                                    _a.sent();
                                    _a.label = 3;
                                case 3: return [2 /*return*/];
                            }
                        });
                    });
                },
            };
        };
        /**
         * Execute benchmarks parallelly.
         *
         * @param context    The context for execute.
         * @param callbacks  Callback functions. same as callbacks of {@link Suite.run}.
         *
         * @return  Result of benchmarks.
         */
        Suite.prototype._runParallel = function (context, callbacks) {
            return __awaiter(this, void 0, void 0, function () {
                var results, _a, _b, _c;
                var _this = this;
                return __generator(this, function (_d) {
                    switch (_d.label) {
                        case 0: return [4 /*yield*/, this.before.call(context)];
                        case 1:
                            _d.sent();
                            _b = (_a = [].concat).apply;
                            _c = [[]];
                            return [4 /*yield*/, Promise.all(this.benchmarks.map(function (x, i) { return __awaiter(_this, void 0, void 0, function () {
                                    var ctx, result;
                                    return __generator(this, function (_a) {
                                        switch (_a.label) {
                                            case 0:
                                                ctx = __assign({}, context);
                                                return [4 /*yield*/, this.beforeEach.call(ctx, i, x)];
                                            case 1:
                                                _a.sent();
                                                return [4 /*yield*/, x.run(ctx, this._makeCallbacks(i, callbacks))];
                                            case 2:
                                                result = _a.sent();
                                                return [4 /*yield*/, this.afterEach.call(ctx, i, x, result)];
                                            case 3:
                                                _a.sent();
                                                return [2 /*return*/, result];
                                        }
                                    });
                                }); }))];
                        case 2:
                            results = _b.apply(_a, _c.concat([_d.sent()]));
                            return [4 /*yield*/, this.after.call(context, results)];
                        case 3:
                            _d.sent();
                            return [2 /*return*/, results];
                    }
                });
            });
        };
        /**
         * Execute benchmarks sequential.
         *
         * @param context    The context for execute.
         * @param callbacks  Callback functions. same as callbacks of {@link Suite.run}.
         *
         * @return  Result of benchmarks.
         */
        Suite.prototype._runSequential = function (context, callbacks) {
            return __awaiter(this, void 0, void 0, function () {
                var results, i, b, ctx, result;
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: return [4 /*yield*/, this.before.call(context)];
                        case 1:
                            _a.sent();
                            results = [];
                            i = 0;
                            _a.label = 2;
                        case 2:
                            if (!(i < this.benchmarks.length)) return [3 /*break*/, 7];
                            b = this.benchmarks[i];
                            ctx = __assign({}, context);
                            return [4 /*yield*/, this.beforeEach.call(ctx, i, b)];
                        case 3:
                            _a.sent();
                            return [4 /*yield*/, b.run(ctx, this._makeCallbacks(i, callbacks))];
                        case 4:
                            result = _a.sent();
                            results.push(result);
                            return [4 /*yield*/, this.afterEach.call(ctx, i, b, result)];
                        case 5:
                            _a.sent();
                            _a.label = 6;
                        case 6:
                            i += 1;
                            return [3 /*break*/, 2];
                        case 7: 
                        /* eslint-enable no-await-in-loop */
                        return [4 /*yield*/, this.after.call(context, results)];
                        case 8:
                            /* eslint-enable no-await-in-loop */
                            _a.sent();
                            return [2 /*return*/, results];
                    }
                });
            });
        };
        /**
         * Execute benchmarks in this suite.
         *
         * All benchmarks will execute parallel if enabled {@link Suite.parallel} option.
         * Else do execute sequentially by added order.
         *
         * @param context    The `this` for each benchmarking functions.
         *                   `__proto__` will override with this instance.
         * @param callbacks  Callback functions.
         *
         * @return  An array of {@link Result}s.
         */
        Suite.prototype.run = function (context, callbacks) {
            if (context === void 0) { context = {}; }
            if (callbacks === void 0) { callbacks = {}; }
            return __awaiter(this, void 0, void 0, function () {
                var ctx;
                return __generator(this, function (_a) {
                    ctx = __assign({}, context);
                    ctx.__proto__ = this;
                    if (this.parallel) {
                        return [2 /*return*/, this._runParallel(ctx, callbacks)];
                    }
                    return [2 /*return*/, this._runSequential(ctx, callbacks)];
                });
            });
        };
        return Suite;
    }());

    exports.AsyncMarkAssertionError = AsyncMarkAssertionError;
    exports.Benchmark = Benchmark;
    exports.Result = Result;
    exports.Suite = Suite;
    exports.timeit = timeit;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=index.js.map
